#!/usr/bin/env python
import unittest
import copy
import json
import os
import manifest
from intervention_test import TestIntervention

import emodpy_tbhiv.interventions.sub_clinical_test as sct
import emod_api.campaign as camp


class Testsubclin(TestIntervention):
    """
    Test DiagnosticSubClinical intervention.
    This test is skipped due to https://github.com/InstituteforDiseaseModeling/emodpy-tbhiv/issues/11
    """
    def skip_test_default(self):
        super().trigger_treatment_list_test(sct.DiagnosticSubClinical, "DiagnosticSubClinical", "camp_subclin.json")

    def skip_test_all(self):
        sct.schema_path = self.schema
        trigger_treatment_list = ['infected']
        base_sensitivity_active_no_HIV = 0.1
        base_specificity = 0.9
        cd4_strata = [110.0, 205.0, 900.0]
        strata_sensitivity = [0.5, 0.4, 0.1]
        subclinical_only = 0.8
        pos_event = 'LAM_Positive'
        neg_event = 'LAM_Negative'
        def_event = 'LAM_Default'
        treatment_fraction = 0.2
        start_day = 1
        duration = 5
        property_restrictions_list = ['Demo: 9']
        expected_property_restrictions_list = copy.deepcopy(property_restrictions_list)
        nodeIDs = []
        black_period = 0
        black_trigger = 'Blackout'
        event_name = 'TB Diagnosis Active Simple'
        event = sct.DiagnosticSubClinical(trigger_treatment_list, base_sensitivity_active_no_HIV, base_specificity,
                                          cd4_strata, strata_sensitivity, subclinical_only, pos_event, neg_event,
                                          def_event, treatment_fraction, start_day, duration, property_restrictions_list,
                                          nodeIDs, black_period, black_trigger, event_name)
        camp.add(event, first=True)
        camp_filename = os.path.join(self.output, "camp_subclin.json")
        manifest.delete_existing_file(camp_filename)
        camp.save(camp_filename)
        self.assertTrue(os.path.isfile(camp_filename))

        with open(camp_filename, 'r') as file:
            camp_file = json.load(file)
        ecc = camp_file["Events"][0]["Event_Coordinator_Config"]
        ic = ecc["Intervention_Config"]
        intervention = ic["Actual_IndividualIntervention_Config"]

        self.assertEqual(intervention["Base_Sensitivity"], base_sensitivity_active_no_HIV)
        self.assertEqual(intervention["Base_Specificity"], base_specificity)
        self.assertEqual(intervention["Event_Or_Config"], "Event")
        self.assertEqual(intervention["Positive_Diagnosis_Event"], pos_event)
        self.assertEqual(intervention["Negative_Diagnosis_Event"], neg_event)
        self.assertEqual(intervention["Defaulters_Event"], def_event)
        self.assertEqual(intervention["Treatment_Fraction"], treatment_fraction)

        # Todo: add assertions for cd4_strata, strata_sensitivity and subclinical_only

        self.assertEqual(ic["Property_Restrictions"], expected_property_restrictions_list)
        self.assertEqual(ic["Duration"], duration)
        self.assertEqual(ic["Blackout_Period"], black_period)
        self.assertEqual(ic["Blackout_Event_Trigger"], black_trigger)
        self.assertEqual(camp_file["Events"][0]["Start_Day"], start_day)
        self.assertEqual(camp.pubsub_signals_pubbing, [pos_event, neg_event, def_event])


if __name__ == '__main__':
    unittest.main()

